@@ -10,5 +10,5 @@
 
 #define QT_FEATURE_wayland_client_qt_shell 1
 
-#define QT_FEATURE_egl_extension_platform_wayland -1
+#define QT_FEATURE_egl_extension_platform_wayland 1
 
