@@ -65,6 +65,7 @@ class QWaylandTextInputMethod;
 #if QT_CONFIG(cursor)
 class QWaylandCursorTheme;
 class QWaylandCursorShape;
+template <typename T>
 class CursorSurface;
 #endif
 
@@ -83,7 +84,9 @@ public:
     QWaylandInputDevice(QWaylandDisplay *display, int version, uint32_t id);
     ~QWaylandInputDevice() override;
 
+    uint32_t id() const { return mId; }
     uint32_t capabilities() const { return mCaps; }
+    QString seatname()  const { return mSeatName; }
 
     QWaylandDisplay *display() const { return mQDisplay; }
     struct ::wl_seat *wl_seat() { return QtWayland::wl_seat::object(); }
@@ -91,6 +94,7 @@ public:
 #if QT_CONFIG(cursor)
     void setCursor(const QCursor *cursor, const QSharedPointer<QWaylandBuffer> &cachedBuffer = {}, int fallbackOutputScale = 1);
 #endif
+    void handleStartDrag();
     void handleEndDrag();
 
 #if QT_CONFIG(wayland_datadevice)
@@ -142,7 +146,9 @@ protected:
     QWaylandDisplay *mQDisplay = nullptr;
     struct wl_display *mDisplay = nullptr;
 
+    uint32_t mId = -1;
     uint32_t mCaps = 0;
+    QString mSeatName;
 
 #if QT_CONFIG(cursor)
     struct CursorState {
@@ -179,6 +185,7 @@ protected:
     uint32_t mSerial = 0;
 
     void seat_capabilities(uint32_t caps) override;
+    void seat_name(const QString &name) override;
     void handleTouchPoint(int id, QEventPoint::State state, const QPointF &surfacePosition = QPoint());
 
     QPointingDevice *mTouchDevice = nullptr;
@@ -189,6 +196,7 @@ protected:
     friend class QWaylandPointerGestureSwipe;
     friend class QWaylandPointerGesturePinch;
     friend class QWaylandWindow;
+    friend class QWaylandTabletToolV2;
 };
 
 inline uint32_t QWaylandInputDevice::serial() const
@@ -230,13 +238,11 @@ public:
     uint32_t mNativeModifiers = 0;
 
     struct repeatKey {
-        int key;
-        uint32_t code;
-        uint32_t time;
+        int key = 0;
+        uint32_t code = 0;
+        uint32_t time = 0 ;
         QString text;
-        Qt::KeyboardModifiers modifiers;
-        uint32_t nativeVirtualKey;
-        uint32_t nativeModifiers;
+        uint32_t nativeVirtualKey = 0;
     } mRepeatKey;
 
     QTimer mRepeatTimer;
@@ -255,7 +261,7 @@ public:
     }
 #endif
 
-private slots:
+private Q_SLOTS:
     void handleFocusDestroyed();
     void handleFocusLost();
 
@@ -287,7 +293,7 @@ public:
     void updateCursor();
     void cursorTimerCallback();
     void cursorFrameCallback();
-    CursorSurface *getOrCreateCursorSurface();
+    CursorSurface<QWaylandInputDevice::Pointer> *getOrCreateCursorSurface();
 #endif
     QWaylandInputDevice *seat() const { return mParent; }
 
@@ -311,7 +317,7 @@ protected:
     void pointer_axis_value120(uint32_t axis, int32_t value120) override;
     void pointer_axis_relative_direction(uint32_t axis, uint32_t direction) override;
 
-private slots:
+private Q_SLOTS:
     void handleFocusDestroyed() { invalidateFocus(); }
 
 private:
@@ -319,6 +325,7 @@ private:
 
 public:
     void releaseButtons();
+    void leavePointers();
 
     QWaylandInputDevice *mParent = nullptr;
     QPointer<QWaylandSurface> mFocus;
@@ -328,7 +335,7 @@ public:
         QScopedPointer<QWaylandCursorShape> shape;
         QWaylandCursorTheme *theme = nullptr;
         int themeBufferScale = 0;
-        QScopedPointer<CursorSurface> surface;
+        QScopedPointer<CursorSurface<QWaylandInputDevice::Pointer>> surface;
         QTimer frameTimer;
         bool gotFrameCallback = false;
         bool gotTimerCallback = false;
@@ -337,10 +344,7 @@ public:
     QPointF mSurfacePos;
     QPointF mGlobalPos;
     Qt::MouseButtons mButtons = Qt::NoButton;
-#if QT_CONFIG(cursor)
-    wl_buffer *mCursorBuffer = nullptr;
-    Qt::CursorShape mCursorShape = Qt::BitmapCursor;
-#endif
+    Qt::MouseButton mLastButton = Qt::NoButton;
 
     struct FrameData {
         QWaylandPointerEvent *event = nullptr;
